require "Window"
require "GameLib"


local FramerateMonitor = {} 
local VILib, GeminiColor
local pairs = pairs
local strAddonName = "Framerate Monitor"
local arSize = {100, 36}
local tVersion = {nMajor = 1, nMinor = 0, nPatch = 3}

local tDefaultSettings = {
	tVersion = {
		nMajor = tVersion.nMajor,
		nMinor = tVersion.nMinor,
		nPatch = tVersion.nPatch,
	},
	bFirstRun = true,
	tThresholds = {
		{20, {255, 0, 0}},
		{30, {255, 255, 0}},
		{60, {0, 255, 0}},
	},
	strTextColor = "ffffffff",
	bShowFps = true,
	bLowerCaseFps = true,
	bColorText = true,
	bTransitionColor = true,
	bIgnoreUIScale = false,
	fOpacity = 1,
	nFontType = 1,
	nFontSize = 2,
	nDecimals = 0,
	fUpdateFrequency = 1,
	tOptionsWindowPos = {},
	tWindowOffsets = {
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
	},
	nMainAnchorID = 4,
}

local arFontSizes = {
	[1] = {
		[1] = 9,
		[2] = 10,
		[3] = 11,
		[4] = 12,
		[5] = 13,
		[6] = 14,
		[7] = 16,
		[8] = 24,
	},
	[2] = {
		[1] = 9,
		[2] = 10,
		[3] = 11,
		[4] = 12,
		[5] = 14,
		[6] = 16,
	},
	[3] = {
		[1] = "",
	},
}

local arFontTypes = {
	[1] = "Header",
	[2] = "Interface",
	[3] = "Pixel",
}


function FramerateMonitor:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function FramerateMonitor:Init()
    Apollo.RegisterAddon(self, true, strAddonName, {"Viper:Lib:VILib-1.9", "GeminiColor"})
end

function FramerateMonitor:OnConfigure()
	self:ShowHideOptions()
end

function FramerateMonitor:OnLoad()
	VILib = Apollo.GetPackage("Viper:Lib:VILib-1.9").tPackage
	GeminiColor = Apollo.GetPackage("GeminiColor").tPackage

	self.settings = VILib:CopyTable(tDefaultSettings)
	self.timerUpdater = nil

	self.xmlDoc = XmlDoc.CreateFromFile("FramerateMonitor.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function FramerateMonitor:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "MainForm", "", self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		self.xmlDoc = nil
		self.wndMainText = self.wndMain:FindChild("Text")

		self.wndOptions, self.wndOptionsList = nil, nil

		self.bMainAnchorShown = false
		self.bOptionsOpen = false
		self.InterfaceMenuRegistered = false
		self.strOutput = "%s fps"

		Apollo.RegisterSlashCommand("vfm", "OnSlashCommand", self)

		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("VFM_ToggleOptions", "ShowHideOptions", self)

		if not self.timerUpdater then self.timerUpdater = ApolloTimer.Create(self.settings.fUpdateFrequency, true, "OnUpdateFramerate", self) end

		self:OnInterfaceMenuListHasLoaded()

		self:SetSettings()
		self:SetPosition()
		self:OnUpdateFramerate()
	end
end

function FramerateMonitor:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Account then return nil end
	self.settings.tVersion = VILib:CopyTable(tVersion)
	return self.settings
end

function FramerateMonitor:OnRestore(eLevel, tData)
	if tData then
		self.settings = VILib:MergeTables(self.settings, tData)
		if self.wndMain then self:SetSettings() end
		if self.timerUpdater then self.timerUpdater:Set(self.settings.fUpdateFrequency, true) end
	end
end

function FramerateMonitor:OnInterfaceMenuListHasLoaded()
	if self.InterfaceMenuRegistered then return end
	self.InterfaceMenuRegistered = true
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", strAddonName, {"VFM_ToggleOptions", "", ""})
end

function FramerateMonitor:OnUpdateFramerate()
	local fps = VILib:Round(GameLib.GetFrameRate(), self.settings.nDecimals)
	local strFps = fps
	if fps > 1000 then strFps = VILib:Round(fps / 1000, 1) .. "k" end
	strFps = string.format(self.strOutput, strFps)
	self.wndMainText:SetText(strFps)
	if self.settings.bColorText then
		local threshold = 3
		for i, v in pairs(self.settings.tThresholds) do
			if fps < v[1] then
				threshold = i
				break
			end
		end
		local color = {r = self.settings.tThresholds[threshold][2][1], g = self.settings.tThresholds[threshold][2][2], b = self.settings.tThresholds[threshold][2][3]}
		if self.settings.bTransitionColor then
			if threshold > 1 then
				local percent = (fps - self.settings.tThresholds[threshold-1][1]) / (self.settings.tThresholds[threshold][1] - self.settings.tThresholds[threshold-1][1])
				if percent > 1 then percent = 1 end
				color = VILib:GetTransitionColorFromRGB(self.settings.tThresholds[threshold-1][2], self.settings.tThresholds[threshold][2], percent)
			else
				color = {r = self.settings.tThresholds[threshold][2][1], g = self.settings.tThresholds[threshold][2][2], b = self.settings.tThresholds[threshold][2][3]}
			end
		end
		color.a = 255
		color = VILib:ConvertARGBColorToHex(color)
		self.wndMainText:SetTextColor(color)
	end
end

function FramerateMonitor:SetSettings()
	self.strOutput = "%s"
	if self.settings.bShowFps then
		if self.settings.bLowerCaseFps then
			self.strOutput = self.strOutput .. " fps"
		else
			self.strOutput = self.strOutput .. " FPS"
		end
	end
	self.wndMain:SetOpacity(self.settings.fOpacity, 100)
	self.wndMainText:SetTextColor(self.settings.strTextColor)
	self.wndMainText:SetFont("CRB_" .. arFontTypes[self.settings.nFontType] .. arFontSizes[self.settings.nFontType][self.settings.nFontSize] .. "_O")
	if self.settings.bIgnoreUIScale then VILib:AdjustForUIScale(self.wndMain) end
	self:OnUpdateFramerate()
end

function FramerateMonitor:SetPosition()
	self.wndMain:SetAnchorPoints(unpack(VILib.tAnchors[self.settings.nMainAnchorID].tPoints))
	self.wndMain:SetAnchorOffsets(self:GetOffsets())
end

function FramerateMonitor:GetOffsets()
	local nOL, nOT, nOR, nOB = unpack(self.settings.tWindowOffsets[self.settings.nMainAnchorID])
	if nOL == 0 and nOR == 0 then
		local nOLM, nORM = VILib.tAnchors[self.settings.nMainAnchorID].tOffsetMults[1], VILib.tAnchors[self.settings.nMainAnchorID].tOffsetMults[3]
		nOL, nOR = VILib:Round(arSize[1] * nOLM), VILib:Round(arSize[1] * nORM)
	end
	if nOT == 0 and nOB == 0 then
		local nOTM, nOBM = VILib.tAnchors[self.settings.nMainAnchorID].tOffsetMults[2], VILib.tAnchors[self.settings.nMainAnchorID].tOffsetMults[4]
		nOT, nOB = VILib:Round(arSize[2] * nOTM), VILib:Round(arSize[2] * nOBM)
	end
	return nOL, nOT, nOR, nOB
end

function FramerateMonitor:OnSlashCommand(strCmd, strParam)
	if strParam ~= "" then
		local tParams = {}
		for w in string.gmatch(strParam, "%S*") do
			if w ~= "" then
				if tonumber(w) then
					table.insert(tParams, tonumber(w))
				else
					table.insert(tParams, string.lower(w))
				end
			end
		end
		if tParams[1] == "reset" or tParams[1] == "defaults" then
			self.settings = VILib:CopyTable(tDefaultSettings)
			VILib:WriteToChat("Options reset to defaults.")
		end
	else
		self:ShowHideOptions()
	end
end


function FramerateMonitor:OnMainWindowMove(wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom)
	self.settings.tWindowOffsets[self.settings.nMainAnchorID] = VILib:Pack(self.wndMain:GetAnchorOffsets())
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetX:Slider"):SetValue(self.settings.tWindowOffsets[self.settings.nMainAnchorID][1])
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetX:Output"):SetText(self.settings.tWindowOffsets[self.settings.nMainAnchorID][1])
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetY:Slider"):SetValue(self.settings.tWindowOffsets[self.settings.nMainAnchorID][2])
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetY:Output"):SetText(self.settings.tWindowOffsets[self.settings.nMainAnchorID][2])
end


---------------------------------------------------------------------------------------------------
-- OptionsList Functions
---------------------------------------------------------------------------------------------------

function FramerateMonitor:SetOptions()
	local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetX:Slider"):SetMinMax(-nScreenWidth, nScreenWidth)
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetY:Slider"):SetMinMax(-nScreenHeight, nScreenHeight)

	self:FillAnchorList()
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetX:Slider"):SetValue(self.settings.tWindowOffsets[self.settings.nMainAnchorID][1])
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetX:Output"):SetText(self.settings.tWindowOffsets[self.settings.nMainAnchorID][1])
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetY:Slider"):SetValue(self.settings.tWindowOffsets[self.settings.nMainAnchorID][2])
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetY:Output"):SetText(self.settings.tWindowOffsets[self.settings.nMainAnchorID][2])

	self.wndOptionsList:FindChild("LookOptionsContainer:FpsShow:Button"):SetCheck(self.settings.bShowFps)
	self.wndOptionsList:FindChild("LookOptionsContainer:LowerCaseFps:Button"):SetCheck(self.settings.bLowerCaseFps)
	self.wndOptionsList:FindChild("LookOptionsContainer:ColorTextUse:Button"):SetCheck(self.settings.bColorText)
	self.wndOptionsList:FindChild("LookOptionsContainer:TransitionColorsUse:Button"):SetCheck(self.settings.bTransitionColor)
	self.wndOptionsList:FindChild("LookOptionsContainer:IgnoreUIScale:Button"):SetCheck(self.settings.bIgnoreUIScale)
	self.wndOptionsList:FindChild("LookOptionsContainer:FontType:Slider"):SetValue(self.settings.nFontType)
	self.wndOptionsList:FindChild("LookOptionsContainer:FontType:Output"):SetText(arFontTypes[self.settings.nFontType])
	self.wndOptionsList:FindChild("LookOptionsContainer:FontSize:Slider"):SetMinMax(1, #arFontSizes[self.settings.nFontType])
	self.wndOptionsList:FindChild("LookOptionsContainer:FontSize:Slider"):SetValue(self.settings.nFontSize)
	self.wndOptionsList:FindChild("LookOptionsContainer:FontSize:Output"):SetText(arFontSizes[self.settings.nFontType][self.settings.nFontSize])
	self.wndOptionsList:FindChild("LookOptionsContainer:TextColor:ColorSwatchBG:ColorSwatch"):SetBGColor(self.settings.strTextColor)
	self.wndOptionsList:FindChild("LookOptionsContainer:Opacity:Slider"):SetValue(VILib:Round(self.settings.fOpacity * 100))
	self.wndOptionsList:FindChild("LookOptionsContainer:Opacity:Output"):SetText(VILib:Round(self.settings.fOpacity * 100) .. "%")
	self.wndOptionsList:FindChild("LookOptionsContainer:UpdateFrequency:Slider"):SetValue(self.settings.fUpdateFrequency)
	self.wndOptionsList:FindChild("LookOptionsContainer:UpdateFrequency:Output"):SetText(VILib:Round(self.settings.fUpdateFrequency, 1, true))

	for i = 1, 3 do
		self.wndOptionsList:FindChild("ThresholdContainer:Threshold" .. i .. ":Slider"):SetValue(self.settings.tThresholds[i][1])
		self.wndOptionsList:FindChild("ThresholdContainer:Threshold" .. i .. ":Output"):SetText(self.settings.tThresholds[i][1])
	end
end

function FramerateMonitor:ShowHideOptions()
	self.bOptionsOpen = not self.bOptionsOpen
	if not self.wndOptions then
		if not self.xmlDocOptions then self.xmlDocOptions = XmlDoc.CreateFromFile("Options.xml") end
		self.wndOptions = Apollo.LoadForm(self.xmlDocOptions, "OptionsForm", nil, self)
		self.wndOptionsList = Apollo.LoadForm(self.xmlDocOptions, "OptionsList", self.wndOptions:FindChild("ContentFrame"), self)
	end
	if self.bOptionsOpen then
		self:SetOptions()
		self.wndOptions:ToFront()
		local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
		local nLeft, nTop, nRight, nBottom = self.wndOptions:GetRect()
		local nWidth, nHeight = nRight - nLeft, nBottom - nTop
		if self.settings.tOptionsWindowPos[1] then
			self.wndOptions:Move(VILib:Round(nScreenWidth * self.settings.tOptionsWindowPos[1]), VILib:Round(nScreenHeight * self.settings.tOptionsWindowPos[2]), nWidth, nHeight)
		else
			self.wndOptions:Move(VILib:Round((nScreenWidth / 2) - (nWidth / 2)), VILib:Round((nScreenHeight / 2) - (nHeight / 2)), nWidth, nHeight)
		end
	end
	self.wndOptions:Show(self.bOptionsOpen, false)

	self.wndMain:SetStyle("Moveable", self.bOptionsOpen)
	self.wndMain:SetStyle("IgnoreMouse", not self.bOptionsOpen)
end

function FramerateMonitor:OnOptionsWindowMove(wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom)
	local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
	local nPosLeft, nPosTop = self.wndOptions:GetPos()
	self.settings.tOptionsWindowPos[1], self.settings.tOptionsWindowPos[2] = nPosLeft / nScreenWidth, nPosTop / nScreenHeight
end

function FramerateMonitor:OnOptionsWindowClosed(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	self:ShowHideOptions()
end

function FramerateMonitor:OnOptionsWindowHide(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	if self.wndOptions then
		self.wndOptionsList:Destroy()
		self.wndOptionsList = nil
		self.wndOptions:Destroy()
		self.wndOptions = nil
	end
end


function FramerateMonitor:OnPosOffsetXSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tWindowOffsets[self.settings.nMainAnchorID][1] = fNewValue
	self.settings.tWindowOffsets[self.settings.nMainAnchorID][3] = fNewValue + arSize[1]
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:SetPosition()
end

function FramerateMonitor:OnPosOffsetYSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.tWindowOffsets[self.settings.nMainAnchorID][2] = fNewValue
	self.settings.tWindowOffsets[self.settings.nMainAnchorID][4] = fNewValue + arSize[2]
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:SetPosition()
end

function FramerateMonitor:OnMainAnchorButtonCheck(wndHandler, wndControl, eMouseButton)
	self.wndOptionsList:FindChild("AnchorParent"):Show(not self.bMainAnchorShown, true)
end

function FramerateMonitor:FillAnchorList()
	local nSel = 0
	self.wndOptionsList:FindChild("AnchorParent:AnchorGrid"):DeleteAll()
	for i, v in ipairs(VILib.tAnchors) do
		if self.settings.nMainAnchorID == i then
			self.wndOptionsList:FindChild("PositionContainer:Anchor:DropdownBtn"):SetText(v["sLabel"])
			nSel = i
		end
		self.wndOptionsList:FindChild("AnchorParent:AnchorGrid"):AddRow(v["sLabel"], "", v)
	end
	self.wndOptionsList:FindChild("AnchorParent:AnchorGrid"):SetCurrentRow(nSel)
end

function FramerateMonitor:OnMainAnchorGridSelChange(wndHandler, wndControl, nRow)
	local tData = wndControl:GetCellData(nRow, 1)
	if not tData then return end
	self.settings.nMainAnchorID = nRow
	local nOL, nOT, nOR, nOB = self:GetOffsets()
	self.wndOptionsList:FindChild("AnchorParent"):Show(false)
	self.wndOptionsList:FindChild("PositionContainer:Anchor:DropdownBtn"):SetText(tData["sLabel"])
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetX:Slider"):SetValue(nOL)
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetX:Output"):SetText(nOL)
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetY:Slider"):SetValue(nOT)
	self.wndOptionsList:FindChild("PositionContainer:PosOffsetY:Output"):SetText(nOT)
	self:SetPosition()
end

function FramerateMonitor:OnMouseDownCatcher(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation)
end

function FramerateMonitor:OnMainAnchorShow(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	self:FillAnchorList()
	self.bMainAnchorShown = true
	self.wndOptionsList:FindChild("PositionContainer:Anchor:DropdownBtnBG"):SetCheck(true)
	self.wndOptionsList:FindChild("PositionContainer:Anchor:DropdownBtn:Arrow"):SetSprite("CRB_Basekit:kitIcon_Holo_UpArrow")
	self.wndOptionsList:SetStyle("Escapable", false)
end

function FramerateMonitor:OnMainAnchorClosed(wndHandler, wndControl)
	if wndHandler ~= wndControl then return end
	self.bMainAnchorShown = false
	self.wndOptionsList:FindChild("PositionContainer:Anchor:DropdownBtnBG"):SetCheck(false)
	self.wndOptionsList:FindChild("PositionContainer:Anchor:DropdownBtn:Arrow"):SetSprite("CRB_Basekit:kitIcon_Holo_DownArrow")
	self.EscapeTimer = ApolloTimer.Create(0.1, false, "OnMainEscapableTimer", self)
end

function FramerateMonitor:OnMainAnchorButtonEnter(wndHandler, wndControl, x, y)
	self.wndOptionsList:FindChild("AnchorParent"):SetStyle("CloseOnExternalClick", false)
end

function FramerateMonitor:OnMainAnchorButtonExit(wndHandler, wndControl, x, y)
	self.wndOptionsList:FindChild("AnchorParent"):SetStyle("CloseOnExternalClick", true)
end

function FramerateMonitor:OnMainEscapableTimer()
	self.wndOptionsList:SetStyle("Escapable", true)
	self:SetSettings()
end


function FramerateMonitor:OnShowFpsButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bShowFps = wndControl:IsChecked()
	self:SetSettings()
end

function FramerateMonitor:OnLowerCaseFpsButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bLowerCaseFps = wndControl:IsChecked()
	self:SetSettings()
end

function FramerateMonitor:OnColorTextUseButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bColorText = wndControl:IsChecked()
	self:SetSettings()
end

function FramerateMonitor:OnColorTransitionsUseButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bTransitionColor = wndControl:IsChecked()
	self:OnUpdateFramerate()
end

function FramerateMonitor:OnIgnoreUIScaleButtonUp(wndHandler, wndControl, eMouseButton)
	self.settings.bIgnoreUIScale = wndControl:IsChecked()
	self:SetSettings()
end

function FramerateMonitor:OnTextColorMouseUp(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY)
	if wndHandler ~= wndControl then return end
	GeminiColor:ShowColorPicker(self, "UpdateTextColor", true, self.settings.sTextColor)
end

function FramerateMonitor:UpdateTextColor(strColor)
	self.settings.strTextColor = strColor
	if self.wndOptionsList then self.wndOptionsList:FindChild("TextColor"):FindChild("ColorSwatch"):SetBGColor(self.settings.strTextColor) end
	self:SetSettings()
end

function FramerateMonitor:OnOpacitySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.fOpacity = fNewValue / 100
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue .. "%")
	self:SetSettings()
end

function FramerateMonitor:OnUpdateFrequencySliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.fUpdateFrequency = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(VILib:Round(fNewValue, 1, true) .. " sec")
	self.timerUpdater:Set(self.settings.fUpdateFrequency, true)
end

function FramerateMonitor:OnFontTypeSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nFontType = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(arFontTypes[fNewValue])
	self.wndOptionsList:FindChild("LookOptionsContainer:FontSize:Slider"):SetMinMax(1, #arFontSizes[fNewValue])
	if self.settings.nFontSize > #arFontSizes[fNewValue] then
		self.settings.nFontSize = #arFontSizes[fNewValue]
		self.wndOptionsList:FindChild("LookOptionsContainer:FontSize:Slider"):SetValue(self.settings.nFontSize)
	end
	self.wndOptionsList:FindChild("LookOptionsContainer:FontSize:Output"):SetText(arFontSizes[fNewValue][self.settings.nFontSize])
	self:SetSettings()
end

function FramerateMonitor:OnFontSizeSliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	self.settings.nFontSize = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(arFontSizes[self.settings.nFontType][fNewValue])
	self:SetSettings()
end

function FramerateMonitor:OnThreshold1SliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	if fNewValue >= self.settings.tThresholds[2][1] then
		wndControl:SetValue(fOldValue)
		return
	end
	self.settings.tThresholds[1][1] = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:OnUpdateFramerate()
end

function FramerateMonitor:OnThreshold2SliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	if fNewValue >= self.settings.tThresholds[3][1] or fNewValue <= self.settings.tThresholds[1][1] then
		wndControl:SetValue(fOldValue)
		return
	end
	self.settings.tThresholds[2][1] = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:OnUpdateFramerate()
end

function FramerateMonitor:OnThreshold3SliderChanged(wndHandler, wndControl, fNewValue, fOldValue)
	if fNewValue <= self.settings.tThresholds[2][1] then
		wndControl:SetValue(fOldValue)
		return
	end
	self.settings.tThresholds[3][1] = fNewValue
	wndControl:GetParent():FindChild("Output"):SetText(fNewValue)
	self:OnUpdateFramerate()
end



local FramerateMonitorInst = FramerateMonitor:new()
FramerateMonitorInst:Init()
